@@ -17,11 +17,11 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.02';
+$VERSION = '1.04';
 
 my $MAX_PACKETS = 2;    # maximum packets to scan from each stream at start of file
 
-# Information types recognizedi in Ogg files
+# Information types recognized in Ogg files
 %Image::ExifTool::Ogg::Main = (
     NOTES => q{
         ExifTool extracts the following types of information from Ogg files.  See
@@ -145,6 +145,7 @@ sub ProcessOGG($$)
         # calculate total data length
         my $dataLen = Get8u(\$buff, 27);
         if ($nseg) {
+            last unless $raf;
             $raf->Read($buff, $nseg-1) == $nseg-1 or last;
             my @segs = unpack('C*', $buff);
             # could check that all these (but the last) are 255...
@@ -160,7 +161,7 @@ sub ProcessOGG($$)
             }
         }
         # read page data
-        $raf->Read($buff, $dataLen) == $dataLen or last;
+        last unless $raf and $raf->Read($buff, $dataLen) == $dataLen;
         if ($verbose > 1) {
             printf $out "Page %d, stream 0x%x, flag 0x%x (%d bytes)\n",
                    $pageNum, $stream, $flag, $dataLen;
@@ -189,7 +190,7 @@ sub ProcessOGG($$)
     if (defined $numFlac and defined $val{$stream}) {
         # process FLAC headers as if it was a complete FLAC file
         require Image::ExifTool::FLAC;
-        my %dirInfo = ( RAF => new File::RandomAccess(\$val{$stream}) );
+        my %dirInfo = ( RAF => File::RandomAccess->new(\$val{$stream}) );
         Image::ExifTool::FLAC::ProcessFLAC($et, \%dirInfo);
     }
     return $success;
@@ -214,7 +215,7 @@ information from Ogg bitstream container files.
 
 =head1 AUTHOR
 
-Copyright 2003-2024, Phil Harvey (philharvey66 at gmail.com)
+Copyright 2003-2026, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
